@@ -6,20 +6,6 @@ import os
 import re
 import time
 import zlib
-from ._util import py3
-
-try:
-    from UserDict import UserDict  # Python 2.x
-except ImportError:
-    UserDict = collections.UserDict  # Python 3.x
-
-
-if py3:  # Python 3.x
-    def make_bytes(s):
-        return s.encode("us-ascii")
-else:  # Python 2.x
-    def make_bytes(s):  # pragma: no cover
-        return s        # pragma: no cover
 
 
 # see 7.9.2.2 Text String Type on page 86 and D.3 PDFDocEncoding Character Set
@@ -29,62 +15,61 @@ def encode_text(s):
 
 
 PDFDocEncoding = {
-    0x16: u"\u0017",
-    0x18: u"\u02D8",
-    0x19: u"\u02C7",
-    0x1A: u"\u02C6",
-    0x1B: u"\u02D9",
-    0x1C: u"\u02DD",
-    0x1D: u"\u02DB",
-    0x1E: u"\u02DA",
-    0x1F: u"\u02DC",
-    0x80: u"\u2022",
-    0x81: u"\u2020",
-    0x82: u"\u2021",
-    0x83: u"\u2026",
-    0x84: u"\u2014",
-    0x85: u"\u2013",
-    0x86: u"\u0192",
-    0x87: u"\u2044",
-    0x88: u"\u2039",
-    0x89: u"\u203A",
-    0x8A: u"\u2212",
-    0x8B: u"\u2030",
-    0x8C: u"\u201E",
-    0x8D: u"\u201C",
-    0x8E: u"\u201D",
-    0x8F: u"\u2018",
-    0x90: u"\u2019",
-    0x91: u"\u201A",
-    0x92: u"\u2122",
-    0x93: u"\uFB01",
-    0x94: u"\uFB02",
-    0x95: u"\u0141",
-    0x96: u"\u0152",
-    0x97: u"\u0160",
-    0x98: u"\u0178",
-    0x99: u"\u017D",
-    0x9A: u"\u0131",
-    0x9B: u"\u0142",
-    0x9C: u"\u0153",
-    0x9D: u"\u0161",
-    0x9E: u"\u017E",
-    0xA0: u"\u20AC",
+    0x16: "\u0017",
+    0x18: "\u02D8",
+    0x19: "\u02C7",
+    0x1A: "\u02C6",
+    0x1B: "\u02D9",
+    0x1C: "\u02DD",
+    0x1D: "\u02DB",
+    0x1E: "\u02DA",
+    0x1F: "\u02DC",
+    0x80: "\u2022",
+    0x81: "\u2020",
+    0x82: "\u2021",
+    0x83: "\u2026",
+    0x84: "\u2014",
+    0x85: "\u2013",
+    0x86: "\u0192",
+    0x87: "\u2044",
+    0x88: "\u2039",
+    0x89: "\u203A",
+    0x8A: "\u2212",
+    0x8B: "\u2030",
+    0x8C: "\u201E",
+    0x8D: "\u201C",
+    0x8E: "\u201D",
+    0x8F: "\u2018",
+    0x90: "\u2019",
+    0x91: "\u201A",
+    0x92: "\u2122",
+    0x93: "\uFB01",
+    0x94: "\uFB02",
+    0x95: "\u0141",
+    0x96: "\u0152",
+    0x97: "\u0160",
+    0x98: "\u0178",
+    0x99: "\u017D",
+    0x9A: "\u0131",
+    0x9B: "\u0142",
+    0x9C: "\u0153",
+    0x9D: "\u0161",
+    0x9E: "\u017E",
+    0xA0: "\u20AC",
 }
 
 
 def decode_text(b):
-    if b[:len(codecs.BOM_UTF16_BE)] == codecs.BOM_UTF16_BE:
-        return b[len(codecs.BOM_UTF16_BE):].decode("utf_16_be")
-    elif py3:  # Python 3.x
+    if b[: len(codecs.BOM_UTF16_BE)] == codecs.BOM_UTF16_BE:
+        return b[len(codecs.BOM_UTF16_BE) :].decode("utf_16_be")
+    else:
         return "".join(PDFDocEncoding.get(byte, chr(byte)) for byte in b)
-    else:  # Python 2.x
-        return u"".join(PDFDocEncoding.get(ord(byte), byte) for byte in b)
 
 
 class PdfFormatError(RuntimeError):
     """An error that probably indicates a syntactic or semantic error in the
     PDF file structure"""
+
     pass
 
 
@@ -93,8 +78,9 @@ def check_format_condition(condition, error_message):
         raise PdfFormatError(error_message)
 
 
-class IndirectReference(collections.namedtuple("IndirectReferenceTuple",
-                        ["object_id", "generation"])):
+class IndirectReference(
+    collections.namedtuple("IndirectReferenceTuple", ["object_id", "generation"])
+):
     def __str__(self):
         return "%s %s R" % self
 
@@ -102,9 +88,11 @@ class IndirectReference(collections.namedtuple("IndirectReferenceTuple",
         return self.__str__().encode("us-ascii")
 
     def __eq__(self, other):
-        return other.__class__ is self.__class__ and \
-               other.object_id == self.object_id and \
-               other.generation == self.generation
+        return (
+            other.__class__ is self.__class__
+            and other.object_id == self.object_id
+            and other.generation == self.generation
+        )
 
     def __ne__(self, other):
         return not (self == other)
@@ -120,9 +108,9 @@ class IndirectObjectDef(IndirectReference):
 
 class XrefTable:
     def __init__(self):
-        self.existing_entries = {}          # object ID => (offset, generation)
-        self.new_entries = {}               # object ID => (offset, generation)
-        self.deleted_entries = {0: 65536}   # object ID => generation
+        self.existing_entries = {}  # object ID => (offset, generation)
+        self.new_entries = {}  # object ID => (offset, generation)
+        self.deleted_entries = {0: 65536}  # object ID => generation
         self.reading_finished = False
 
     def __setitem__(self, key, value):
@@ -150,26 +138,27 @@ class XrefTable:
         elif key in self.deleted_entries:
             generation = self.deleted_entries[key]
         else:
-            raise IndexError("object ID " + str(key) +
-                             " cannot be deleted because it doesn't exist")
+            raise IndexError(
+                "object ID " + str(key) + " cannot be deleted because it doesn't exist"
+            )
 
     def __contains__(self, key):
         return key in self.existing_entries or key in self.new_entries
 
     def __len__(self):
-        return len(set(self.existing_entries.keys()) |
-                   set(self.new_entries.keys()) |
-                   set(self.deleted_entries.keys()))
+        return len(
+            set(self.existing_entries.keys())
+            | set(self.new_entries.keys())
+            | set(self.deleted_entries.keys())
+        )
 
     def keys(self):
         return (
-            set(self.existing_entries.keys()) -
-            set(self.deleted_entries.keys())
+            set(self.existing_entries.keys()) - set(self.deleted_entries.keys())
         ) | set(self.new_entries.keys())
 
     def write(self, f):
-        keys = sorted(set(self.new_entries.keys()) |
-                      set(self.deleted_entries.keys()))
+        keys = sorted(set(self.new_entries.keys()) | set(self.deleted_entries.keys()))
         deleted_keys = sorted(set(self.deleted_entries.keys()))
         startxref = f.tell()
         f.write(b"xref\n")
@@ -177,7 +166,7 @@ class XrefTable:
             # find a contiguous sequence of object IDs
             prev = None
             for index, key in enumerate(keys):
-                if prev is None or prev+1 == key:
+                if prev is None or prev + 1 == key:
                     prev = key
                 else:
                     contiguous_keys = keys[:index]
@@ -186,25 +175,25 @@ class XrefTable:
             else:
                 contiguous_keys = keys
                 keys = None
-            f.write(make_bytes("%d %d\n" %
-                               (contiguous_keys[0], len(contiguous_keys))))
+            f.write(b"%d %d\n" % (contiguous_keys[0], len(contiguous_keys)))
             for object_id in contiguous_keys:
                 if object_id in self.new_entries:
-                    f.write(make_bytes("%010d %05d n \n" %
-                                       self.new_entries[object_id]))
+                    f.write(b"%010d %05d n \n" % self.new_entries[object_id])
                 else:
                     this_deleted_object_id = deleted_keys.pop(0)
-                    check_format_condition(object_id == this_deleted_object_id,
-                                           "expected the next deleted object "
-                                           "ID to be %s, instead found %s" %
-                                           (object_id, this_deleted_object_id))
+                    check_format_condition(
+                        object_id == this_deleted_object_id,
+                        f"expected the next deleted object ID to be {object_id}, "
+                        f"instead found {this_deleted_object_id}",
+                    )
                     try:
                         next_in_linked_list = deleted_keys[0]
                     except IndexError:
                         next_in_linked_list = 0
-                    f.write(make_bytes("%010d %05d f \n" %
-                                       (next_in_linked_list,
-                                        self.deleted_entries[object_id])))
+                    f.write(
+                        b"%010d %05d f \n"
+                        % (next_in_linked_list, self.deleted_entries[object_id])
+                    )
         return startxref
 
 
@@ -221,84 +210,67 @@ class PdfName:
         return self.name.decode("us-ascii")
 
     def __eq__(self, other):
-        return (isinstance(other, PdfName) and other.name == self.name) or \
-               other == self.name
+        return (
+            isinstance(other, PdfName) and other.name == self.name
+        ) or other == self.name
 
     def __hash__(self):
         return hash(self.name)
 
     def __repr__(self):
-        return "PdfName(%s)" % repr(self.name)
+        return f"PdfName({repr(self.name)})"
 
     @classmethod
     def from_pdf_stream(cls, data):
         return cls(PdfParser.interpret_name(data))
 
-    allowed_chars = set(range(33, 127)) - set(ord(c) for c in "#%/()<>[]{}")
+    allowed_chars = set(range(33, 127)) - {ord(c) for c in "#%/()<>[]{}"}
 
     def __bytes__(self):
         result = bytearray(b"/")
         for b in self.name:
-            if py3:  # Python 3.x
-                if b in self.allowed_chars:
-                    result.append(b)
-                else:
-                    result.extend(make_bytes("#%02X" % b))
-            else:  # Python 2.x
-                if ord(b) in self.allowed_chars:
-                    result.append(b)
-                else:
-                    result.extend(b"#%02X" % ord(b))
+            if b in self.allowed_chars:
+                result.append(b)
+            else:
+                result.extend(b"#%02X" % b)
         return bytes(result)
 
-    __str__ = __bytes__
-
 
 class PdfArray(list):
     def __bytes__(self):
         return b"[ " + b" ".join(pdf_repr(x) for x in self) + b" ]"
 
-    __str__ = __bytes__
 
-
-class PdfDict(UserDict):
+class PdfDict(collections.UserDict):
     def __setattr__(self, key, value):
         if key == "data":
-            if hasattr(UserDict, "__setattr__"):
-                UserDict.__setattr__(self, key, value)
-            else:
-                self.__dict__[key] = value
+            collections.UserDict.__setattr__(self, key, value)
         else:
-            if isinstance(key, str):
-                key = key.encode("us-ascii")
-            self[key] = value
+            self[key.encode("us-ascii")] = value
 
     def __getattr__(self, key):
         try:
-            value = self[key]
-        except KeyError:
-            try:
-                value = self[key.encode("us-ascii")]
-            except KeyError:
-                raise AttributeError(key)
+            value = self[key.encode("us-ascii")]
+        except KeyError as e:
+            raise AttributeError(key) from e
         if isinstance(value, bytes):
             value = decode_text(value)
         if key.endswith("Date"):
             if value.startswith("D:"):
                 value = value[2:]
 
-            relationship = 'Z'
+            relationship = "Z"
             if len(value) > 17:
                 relationship = value[14]
                 offset = int(value[15:17]) * 60
                 if len(value) > 20:
                     offset += int(value[18:20])
 
-            format = '%Y%m%d%H%M%S'[:len(value) - 2]
-            value = time.strptime(value[:len(format)+2], format)
-            if relationship in ['+', '-']:
+            format = "%Y%m%d%H%M%S"[: len(value) - 2]
+            value = time.strptime(value[: len(format) + 2], format)
+            if relationship in ["+", "-"]:
                 offset *= 60
-                if relationship == '+':
+                if relationship == "+":
                     offset *= -1
                 value = time.gmtime(calendar.timegm(value) + offset)
         return value
@@ -316,20 +288,13 @@ class PdfDict(UserDict):
         out.extend(b"\n>>")
         return bytes(out)
 
-    if not py3:
-        __str__ = __bytes__
-
 
 class PdfBinary:
     def __init__(self, data):
         self.data = data
 
-    if py3:  # Python 3.x
-        def __bytes__(self):
-            return make_bytes("<%s>" % "".join("%02X" % b for b in self.data))
-    else:  # Python 2.x
-        def __str__(self):
-            return "<%s>" % "".join("%02X" % ord(b) for b in self.data)
+    def __bytes__(self):
+        return b"<%s>" % b"".join(b"%02X" % b for b in self.data)
 
 
 class PdfStream:
@@ -350,8 +315,8 @@ class PdfStream:
             return zlib.decompress(self.buf, bufsize=int(expected_length))
         else:
             raise NotImplementedError(
-                "stream filter %s unknown/unsupported" %
-                repr(self.dictionary.Filter))
+                f"stream filter {repr(self.dictionary.Filter)} unknown/unsupported"
+            )
 
 
 def pdf_repr(x):
@@ -361,19 +326,19 @@ def pdf_repr(x):
         return b"false"
     elif x is None:
         return b"null"
-    elif (isinstance(x, PdfName) or isinstance(x, PdfDict) or
-          isinstance(x, PdfArray) or isinstance(x, PdfBinary)):
+    elif isinstance(x, (PdfName, PdfDict, PdfArray, PdfBinary)):
         return bytes(x)
     elif isinstance(x, int):
         return str(x).encode("us-ascii")
+    elif isinstance(x, float):
+        return str(x).encode("us-ascii")
     elif isinstance(x, time.struct_time):
-        return b'(D:'+time.strftime('%Y%m%d%H%M%SZ', x).encode("us-ascii")+b')'
+        return b"(D:" + time.strftime("%Y%m%d%H%M%SZ", x).encode("us-ascii") + b")"
     elif isinstance(x, dict):
         return bytes(PdfDict(x))
     elif isinstance(x, list):
         return bytes(PdfArray(x))
-    elif ((py3 and isinstance(x, str)) or
-          (not py3 and isinstance(x, unicode))):
+    elif isinstance(x, str):
         return pdf_repr(encode_text(x))
     elif isinstance(x, bytes):
         # XXX escape more chars? handle binary garbage
@@ -386,17 +351,14 @@ def pdf_repr(x):
 
 
 class PdfParser:
-    """Based on https://www.adobe.com/content/dam/acom/en/devnet/acrobat/pdfs/PDF32000_2008.pdf
+    """Based on
+    https://www.adobe.com/content/dam/acom/en/devnet/acrobat/pdfs/PDF32000_2008.pdf
     Supports PDF up to 1.4
     """
 
-    def __init__(self, filename=None, f=None,
-                 buf=None, start_offset=0, mode="rb"):
-        # type: (PdfParser, str, file, Union[bytes, bytearray], int, str)
-        #       -> None
+    def __init__(self, filename=None, f=None, buf=None, start_offset=0, mode="rb"):
         if buf and f:
-            raise RuntimeError(
-                "specify buf or f or filename, but not both buf and f")
+            raise RuntimeError("specify buf or f or filename, but not both buf and f")
         self.filename = filename
         self.buf = buf
         self.f = f
@@ -463,20 +425,20 @@ class PdfParser:
         self.f.write(b"%PDF-1.4\n")
 
     def write_comment(self, s):
-        self.f.write(("%% %s\n" % (s,)).encode("utf-8"))
+        self.f.write(f"% {s}\n".encode())
 
     def write_catalog(self):
         self.del_root()
         self.root_ref = self.next_object_id(self.f.tell())
         self.pages_ref = self.next_object_id(0)
         self.rewrite_pages()
-        self.write_obj(self.root_ref,
-                       Type=PdfName(b"Catalog"),
-                       Pages=self.pages_ref)
-        self.write_obj(self.pages_ref,
-                       Type=PdfName(b"Pages"),
-                       Count=len(self.pages),
-                       Kids=self.pages)
+        self.write_obj(self.root_ref, Type=PdfName(b"Catalog"), Pages=self.pages_ref)
+        self.write_obj(
+            self.pages_ref,
+            Type=PdfName(b"Pages"),
+            Count=len(self.pages),
+            Kids=self.pages,
+        )
         return self.root_ref
 
     def rewrite_pages(self):
@@ -522,8 +484,11 @@ class PdfParser:
         if self.info:
             trailer_dict[b"Info"] = self.info_ref
         self.last_xref_section_offset = start_xref
-        self.f.write(b"trailer\n" + bytes(PdfDict(trailer_dict)) +
-                     make_bytes("\nstartxref\n%d\n%%%%EOF" % start_xref))
+        self.f.write(
+            b"trailer\n"
+            + bytes(PdfDict(trailer_dict))
+            + b"\nstartxref\n%d\n%%%%EOF" % start_xref
+        )
 
     def write_page(self, ref, *objs, **dict_obj):
         if isinstance(ref, int):
@@ -585,12 +550,14 @@ class PdfParser:
         else:
             self.info = PdfDict(self.read_indirect(self.info_ref))
         check_format_condition(b"Type" in self.root, "/Type missing in Root")
-        check_format_condition(self.root[b"Type"] == b"Catalog",
-                               "/Type in Root is not /Catalog")
+        check_format_condition(
+            self.root[b"Type"] == b"Catalog", "/Type in Root is not /Catalog"
+        )
         check_format_condition(b"Pages" in self.root, "/Pages missing in Root")
-        check_format_condition(isinstance(self.root[b"Pages"],
-                               IndirectReference),
-                               "/Pages in Root is not an indirect reference")
+        check_format_condition(
+            isinstance(self.root[b"Pages"], IndirectReference),
+            "/Pages in Root is not an indirect reference",
+        )
         self.pages_ref = self.root[b"Pages"]
         self.page_tree_root = self.read_indirect(self.pages_ref)
         self.pages = self.linearize_page_tree(self.page_tree_root)
@@ -609,22 +576,45 @@ class PdfParser:
             self.xref_table[reference.object_id] = (offset, 0)
         return reference
 
-    delimiter = br"[][()<>{}/%]"
-    delimiter_or_ws = br"[][()<>{}/%\000\011\012\014\015\040]"
-    whitespace = br"[\000\011\012\014\015\040]"
-    whitespace_or_hex = br"[\000\011\012\014\015\0400-9a-fA-F]"
+    delimiter = rb"[][()<>{}/%]"
+    delimiter_or_ws = rb"[][()<>{}/%\000\011\012\014\015\040]"
+    whitespace = rb"[\000\011\012\014\015\040]"
+    whitespace_or_hex = rb"[\000\011\012\014\015\0400-9a-fA-F]"
     whitespace_optional = whitespace + b"*"
     whitespace_mandatory = whitespace + b"+"
-    newline_only = br"[\r\n]+"
-    newline = whitespace_optional + newline_only + whitespace_optional
+    # No "\012" aka "\n" or "\015" aka "\r":
+    whitespace_optional_no_nl = rb"[\000\011\014\040]*"
+    newline_only = rb"[\r\n]+"
+    newline = whitespace_optional_no_nl + newline_only + whitespace_optional_no_nl
     re_trailer_end = re.compile(
-        whitespace_mandatory + br"trailer" + whitespace_optional +
-        br"\<\<(.*\>\>)" + newline + br"startxref" + newline + br"([0-9]+)" +
-        newline + br"%%EOF" + whitespace_optional + br"$", re.DOTALL)
+        whitespace_mandatory
+        + rb"trailer"
+        + whitespace_optional
+        + rb"\<\<(.*\>\>)"
+        + newline
+        + rb"startxref"
+        + newline
+        + rb"([0-9]+)"
+        + newline
+        + rb"%%EOF"
+        + whitespace_optional
+        + rb"$",
+        re.DOTALL,
+    )
     re_trailer_prev = re.compile(
-        whitespace_optional + br"trailer" + whitespace_optional +
-        br"\<\<(.*?\>\>)" + newline + br"startxref" + newline + br"([0-9]+)" +
-        newline + br"%%EOF" + whitespace_optional, re.DOTALL)
+        whitespace_optional
+        + rb"trailer"
+        + whitespace_optional
+        + rb"\<\<(.*?\>\>)"
+        + newline
+        + rb"startxref"
+        + newline
+        + rb"([0-9]+)"
+        + newline
+        + rb"%%EOF"
+        + whitespace_optional,
+        re.DOTALL,
+    )
 
     def read_trailer(self):
         search_start_offset = len(self.buf) - 16384
@@ -636,7 +626,7 @@ class PdfParser:
         last_match = m
         while m:
             last_match = m
-            m = self.re_trailer_end.search(self.buf, m.start()+16)
+            m = self.re_trailer_end.search(self.buf, m.start() + 16)
         if not m:
             m = last_match
         trailer_data = m.group(1)
@@ -648,26 +638,29 @@ class PdfParser:
             self.read_prev_trailer(self.trailer_dict[b"Prev"])
 
     def read_prev_trailer(self, xref_section_offset):
-        trailer_offset = self.read_xref_table(
-            xref_section_offset=xref_section_offset)
+        trailer_offset = self.read_xref_table(xref_section_offset=xref_section_offset)
         m = self.re_trailer_prev.search(
-            self.buf[trailer_offset:trailer_offset+16384])
+            self.buf[trailer_offset : trailer_offset + 16384]
+        )
         check_format_condition(m, "previous trailer not found")
         trailer_data = m.group(1)
-        check_format_condition(int(m.group(2)) == xref_section_offset,
-                               "xref section offset in previous trailer "
-                               "doesn't match what was expected")
+        check_format_condition(
+            int(m.group(2)) == xref_section_offset,
+            "xref section offset in previous trailer doesn't match what was expected",
+        )
         trailer_dict = self.interpret_trailer(trailer_data)
         if b"Prev" in trailer_dict:
             self.read_prev_trailer(trailer_dict[b"Prev"])
 
     re_whitespace_optional = re.compile(whitespace_optional)
     re_name = re.compile(
-        whitespace_optional + br"/([!-$&'*-.0-;=?-Z\\^-z|~]+)(?=" +
-        delimiter_or_ws + br")")
-    re_dict_start = re.compile(whitespace_optional + br"\<\<")
-    re_dict_end = re.compile(
-        whitespace_optional + br"\>\>" + whitespace_optional)
+        whitespace_optional
+        + rb"/([!-$&'*-.0-;=?-Z\\^-z|~]+)(?="
+        + delimiter_or_ws
+        + rb")"
+    )
+    re_dict_start = re.compile(whitespace_optional + rb"\<\<")
+    re_dict_end = re.compile(whitespace_optional + rb"\>\>" + whitespace_optional)
 
     @classmethod
     def interpret_trailer(cls, trailer_data):
@@ -679,30 +672,31 @@ class PdfParser:
                 m = cls.re_dict_end.match(trailer_data, offset)
                 check_format_condition(
                     m and m.end() == len(trailer_data),
-                    "name not found in trailer, remaining data: " +
-                    repr(trailer_data[offset:]))
+                    "name not found in trailer, remaining data: "
+                    + repr(trailer_data[offset:]),
+                )
                 break
             key = cls.interpret_name(m.group(1))
             value, offset = cls.get_value(trailer_data, m.end())
             trailer[key] = value
         check_format_condition(
             b"Size" in trailer and isinstance(trailer[b"Size"], int),
-            "/Size not in trailer or not an integer")
+            "/Size not in trailer or not an integer",
+        )
         check_format_condition(
-            b"Root" in trailer and
-            isinstance(trailer[b"Root"], IndirectReference),
-            "/Root not in trailer or not an indirect reference")
+            b"Root" in trailer and isinstance(trailer[b"Root"], IndirectReference),
+            "/Root not in trailer or not an indirect reference",
+        )
         return trailer
 
-    re_hashes_in_name = re.compile(br"([^#]*)(#([0-9a-fA-F]{2}))?")
+    re_hashes_in_name = re.compile(rb"([^#]*)(#([0-9a-fA-F]{2}))?")
 
     @classmethod
     def interpret_name(cls, raw, as_text=False):
         name = b""
         for m in cls.re_hashes_in_name.finditer(raw):
             if m.group(3):
-                name += m.group(1) + \
-                    bytearray.fromhex(m.group(3).decode("us-ascii"))
+                name += m.group(1) + bytearray.fromhex(m.group(3).decode("us-ascii"))
             else:
                 name += m.group(1)
         if as_text:
@@ -710,37 +704,54 @@ class PdfParser:
         else:
             return bytes(name)
 
-    re_null = re.compile(
-        whitespace_optional + br"null(?=" + delimiter_or_ws + br")")
-    re_true = re.compile(
-        whitespace_optional + br"true(?=" + delimiter_or_ws + br")")
-    re_false = re.compile(
-        whitespace_optional + br"false(?=" + delimiter_or_ws + br")")
+    re_null = re.compile(whitespace_optional + rb"null(?=" + delimiter_or_ws + rb")")
+    re_true = re.compile(whitespace_optional + rb"true(?=" + delimiter_or_ws + rb")")
+    re_false = re.compile(whitespace_optional + rb"false(?=" + delimiter_or_ws + rb")")
     re_int = re.compile(
-        whitespace_optional + br"([-+]?[0-9]+)(?=" + delimiter_or_ws + br")")
+        whitespace_optional + rb"([-+]?[0-9]+)(?=" + delimiter_or_ws + rb")"
+    )
     re_real = re.compile(
-        whitespace_optional + br"([-+]?([0-9]+\.[0-9]*|[0-9]*\.[0-9]+))(?=" +
-        delimiter_or_ws + br")")
-    re_array_start = re.compile(whitespace_optional + br"\[")
-    re_array_end = re.compile(whitespace_optional + br"]")
+        whitespace_optional
+        + rb"([-+]?([0-9]+\.[0-9]*|[0-9]*\.[0-9]+))(?="
+        + delimiter_or_ws
+        + rb")"
+    )
+    re_array_start = re.compile(whitespace_optional + rb"\[")
+    re_array_end = re.compile(whitespace_optional + rb"]")
     re_string_hex = re.compile(
-        whitespace_optional + br"\<(" + whitespace_or_hex + br"*)\>")
-    re_string_lit = re.compile(whitespace_optional + br"\(")
+        whitespace_optional + rb"\<(" + whitespace_or_hex + rb"*)\>"
+    )
+    re_string_lit = re.compile(whitespace_optional + rb"\(")
     re_indirect_reference = re.compile(
-        whitespace_optional + br"([-+]?[0-9]+)" + whitespace_mandatory +
-        br"([-+]?[0-9]+)" + whitespace_mandatory + br"R(?=" + delimiter_or_ws +
-        br")")
+        whitespace_optional
+        + rb"([-+]?[0-9]+)"
+        + whitespace_mandatory
+        + rb"([-+]?[0-9]+)"
+        + whitespace_mandatory
+        + rb"R(?="
+        + delimiter_or_ws
+        + rb")"
+    )
     re_indirect_def_start = re.compile(
-        whitespace_optional + br"([-+]?[0-9]+)" + whitespace_mandatory +
-        br"([-+]?[0-9]+)" + whitespace_mandatory + br"obj(?=" +
-        delimiter_or_ws + br")")
+        whitespace_optional
+        + rb"([-+]?[0-9]+)"
+        + whitespace_mandatory
+        + rb"([-+]?[0-9]+)"
+        + whitespace_mandatory
+        + rb"obj(?="
+        + delimiter_or_ws
+        + rb")"
+    )
     re_indirect_def_end = re.compile(
-        whitespace_optional + br"endobj(?=" + delimiter_or_ws + br")")
+        whitespace_optional + rb"endobj(?=" + delimiter_or_ws + rb")"
+    )
     re_comment = re.compile(
-        br"(" + whitespace_optional + br"%[^\r\n]*" + newline + br")*")
-    re_stream_start = re.compile(whitespace_optional + br"stream\r?\n")
+        rb"(" + whitespace_optional + rb"%[^\r\n]*" + newline + rb")*"
+    )
+    re_stream_start = re.compile(whitespace_optional + rb"stream\r?\n")
     re_stream_end = re.compile(
-        whitespace_optional + br"endstream(?=" + delimiter_or_ws + br")")
+        whitespace_optional + rb"endstream(?=" + delimiter_or_ws + rb")"
+    )
 
     @classmethod
     def get_value(cls, data, offset, expect_indirect=None, max_nesting=-1):
@@ -753,32 +764,37 @@ class PdfParser:
         if m:
             check_format_condition(
                 int(m.group(1)) > 0,
-                "indirect object definition: object ID must be greater than 0")
+                "indirect object definition: object ID must be greater than 0",
+            )
             check_format_condition(
                 int(m.group(2)) >= 0,
-                "indirect object definition: generation must be non-negative")
+                "indirect object definition: generation must be non-negative",
+            )
             check_format_condition(
-                expect_indirect is None or expect_indirect ==
-                IndirectReference(int(m.group(1)), int(m.group(2))),
-                "indirect object definition different than expected")
-            object, offset = cls.get_value(
-                data, m.end(), max_nesting=max_nesting-1)
+                expect_indirect is None
+                or expect_indirect
+                == IndirectReference(int(m.group(1)), int(m.group(2))),
+                "indirect object definition different than expected",
+            )
+            object, offset = cls.get_value(data, m.end(), max_nesting=max_nesting - 1)
             if offset is None:
                 return object, None
             m = cls.re_indirect_def_end.match(data, offset)
-            check_format_condition(
-                m, "indirect object definition end not found")
+            check_format_condition(m, "indirect object definition end not found")
             return object, m.end()
         check_format_condition(
-            not expect_indirect, "indirect object definition not found")
+            not expect_indirect, "indirect object definition not found"
+        )
         m = cls.re_indirect_reference.match(data, offset)
         if m:
             check_format_condition(
                 int(m.group(1)) > 0,
-                "indirect object reference: object ID must be greater than 0")
+                "indirect object reference: object ID must be greater than 0",
+            )
             check_format_condition(
                 int(m.group(2)) >= 0,
-                "indirect object reference: generation must be non-negative")
+                "indirect object reference: generation must be non-negative",
+            )
             return IndirectReference(int(m.group(1)), int(m.group(2))), m.end()
         m = cls.re_dict_start.match(data, offset)
         if m:
@@ -786,12 +802,10 @@ class PdfParser:
             result = {}
             m = cls.re_dict_end.match(data, offset)
             while not m:
-                key, offset = cls.get_value(
-                    data, offset, max_nesting=max_nesting-1)
+                key, offset = cls.get_value(data, offset, max_nesting=max_nesting - 1)
                 if offset is None:
                     return result, None
-                value, offset = cls.get_value(
-                    data, offset, max_nesting=max_nesting-1)
+                value, offset = cls.get_value(data, offset, max_nesting=max_nesting - 1)
                 result[key] = value
                 if offset is None:
                     return result, None
@@ -801,11 +815,12 @@ class PdfParser:
             if m:
                 try:
                     stream_len = int(result[b"Length"])
-                except (TypeError, KeyError, ValueError):
+                except (TypeError, KeyError, ValueError) as e:
                     raise PdfFormatError(
-                        "bad or missing Length in stream dict (%r)" %
-                        result.get(b"Length", None))
-                stream_data = data[m.end():m.end() + stream_len]
+                        "bad or missing Length in stream dict (%r)"
+                        % result.get(b"Length", None)
+                    ) from e
+                stream_data = data[m.end() : m.end() + stream_len]
                 m = cls.re_stream_end.match(data, m.end() + stream_len)
                 check_format_condition(m, "stream end not found")
                 offset = m.end()
@@ -819,8 +834,7 @@ class PdfParser:
             result = []
             m = cls.re_array_end.match(data, offset)
             while not m:
-                value, offset = cls.get_value(
-                    data, offset, max_nesting=max_nesting-1)
+                value, offset = cls.get_value(data, offset, max_nesting=max_nesting - 1)
                 result.append(value)
                 if offset is None:
                     return result, None
@@ -848,10 +862,9 @@ class PdfParser:
         m = cls.re_string_hex.match(data, offset)
         if m:
             # filter out whitespace
-            hex_string = bytearray([
-                b for b in m.group(1)
-                if b in b"0123456789abcdefABCDEF"
-            ])
+            hex_string = bytearray(
+                b for b in m.group(1) if b in b"0123456789abcdefABCDEF"
+            )
             if len(hex_string) % 2 == 1:
                 # append a 0 if the length is not even - yes, at the end
                 hex_string.append(ord(b"0"))
@@ -860,10 +873,11 @@ class PdfParser:
         if m:
             return cls.get_literal_string(data, m.end())
         # return None, offset  # fallback (only for debugging)
-        raise PdfFormatError(
-            "unrecognized object: " + repr(data[offset:offset+32]))
+        raise PdfFormatError("unrecognized object: " + repr(data[offset : offset + 32]))
 
-    re_lit_str_token = re.compile(br"(\\[nrtbf()\\])|(\\[0-9]{1,3})|(\\(\r\n|\r|\n))|(\r\n|\r|\n)|(\()|(\))")
+    re_lit_str_token = re.compile(
+        rb"(\\[nrtbf()\\])|(\\[0-9]{1,3})|(\\(\r\n|\r|\n))|(\r\n|\r|\n)|(\()|(\))"
+    )
     escaped_chars = {
         b"n": b"\n",
         b"r": b"\r",
@@ -881,14 +895,14 @@ class PdfParser:
         ord(b"("): b"(",
         ord(b")"): b")",
         ord(b"\\"): b"\\",
-        }
+    }
 
     @classmethod
     def get_literal_string(cls, data, offset):
         nesting_depth = 0
         result = bytearray()
         for m in cls.re_lit_str_token.finditer(data, offset):
-            result.extend(data[offset:m.start()])
+            result.extend(data[offset : m.start()])
             if m.group(1):
                 result.extend(cls.escaped_chars[m.group(1)[1]])
             elif m.group(2):
@@ -908,30 +922,36 @@ class PdfParser:
             offset = m.end()
         raise PdfFormatError("unfinished literal string")
 
-    re_xref_section_start = re.compile(
-        whitespace_optional + br"xref" + newline)
+    re_xref_section_start = re.compile(whitespace_optional + rb"xref" + newline)
     re_xref_subsection_start = re.compile(
-        whitespace_optional + br"([0-9]+)" + whitespace_mandatory +
-        br"([0-9]+)" + whitespace_optional + newline_only)
-    re_xref_entry = re.compile(br"([0-9]{10}) ([0-9]{5}) ([fn])( \r| \n|\r\n)")
+        whitespace_optional
+        + rb"([0-9]+)"
+        + whitespace_mandatory
+        + rb"([0-9]+)"
+        + whitespace_optional
+        + newline_only
+    )
+    re_xref_entry = re.compile(rb"([0-9]{10}) ([0-9]{5}) ([fn])( \r| \n|\r\n)")
 
     def read_xref_table(self, xref_section_offset):
         subsection_found = False
         m = self.re_xref_section_start.match(
-            self.buf, xref_section_offset + self.start_offset)
+            self.buf, xref_section_offset + self.start_offset
+        )
         check_format_condition(m, "xref section start not found")
         offset = m.end()
         while True:
             m = self.re_xref_subsection_start.match(self.buf, offset)
             if not m:
                 check_format_condition(
-                    subsection_found, "xref subsection start not found")
+                    subsection_found, "xref subsection start not found"
+                )
                 break
             subsection_found = True
             offset = m.end()
             first_object = int(m.group(1))
             num_objects = int(m.group(2))
-            for i in range(first_object, first_object+num_objects):
+            for i in range(first_object, first_object + num_objects):
                 m = self.re_xref_entry.match(self.buf, offset)
                 check_format_condition(m, "xref entry not found")
                 offset = m.end()
@@ -940,9 +960,9 @@ class PdfParser:
                 if not is_free:
                     new_entry = (int(m.group(1)), generation)
                     check_format_condition(
-                        i not in self.xref_table or
-                        self.xref_table[i] == new_entry,
-                        "xref entry duplicated (and not identical)")
+                        i not in self.xref_table or self.xref_table[i] == new_entry,
+                        "xref entry duplicated (and not identical)",
+                    )
                     self.xref_table[i] = new_entry
         return offset
 
@@ -950,12 +970,15 @@ class PdfParser:
         offset, generation = self.xref_table[ref[0]]
         check_format_condition(
             generation == ref[1],
-            "expected to find generation %s for object ID %s in xref table, "
-            "instead found generation %s at offset %s"
-            % (ref[1], ref[0], generation, offset))
-        value = self.get_value(self.buf, offset + self.start_offset,
-                               expect_indirect=IndirectReference(*ref),
-                               max_nesting=max_nesting)[0]
+            f"expected to find generation {ref[1]} for object ID {ref[0]} in xref "
+            f"table, instead found generation {generation} at offset {offset}",
+        )
+        value = self.get_value(
+            self.buf,
+            offset + self.start_offset,
+            expect_indirect=IndirectReference(*ref),
+            max_nesting=max_nesting,
+        )[0]
         self.cached_objects[ref] = value
         return value
 
@@ -963,7 +986,8 @@ class PdfParser:
         if node is None:
             node = self.page_tree_root
         check_format_condition(
-            node[b"Type"] == b"Pages", "/Type of page tree node is not /Pages")
+            node[b"Type"] == b"Pages", "/Type of page tree node is not /Pages"
+        )
         pages = []
         for kid in node[b"Kids"]:
             kid_object = self.read_indirect(kid)
